@@ -1,7 +1,7 @@
 """
 CutoML - A lightweight automl framework for classification and regression tasks.
 
-Copyright (C) 2021 Omkar Udawant
+Copyright (C) 2022 Omkar Udawant
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -16,34 +16,31 @@ GNU General Public License for more details.
 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <https://www.gnu.org/licenses/>.
 """
+import tqdm
+from imblearn.over_sampling import SMOTE
+from pathos.multiprocessing import ProcessingPool
+from sklearn.model_selection import train_test_split
+from sklearn.pipeline import Pipeline
 
-from cutoml.utils import regression_metrics
-from cutoml.utils import classification_metrics
-from cutoml.utils import timer
 from cutoml.config import Classifiers
 from cutoml.config import Regressors
-
-from sklearn.model_selection import train_test_split
-from sklearn.preprocessing import StandardScaler
-from sklearn.preprocessing import MinMaxScaler
-from sklearn.pipeline import Pipeline
-from imblearn.pipeline import Pipeline as imblearn_pipeline
-from imblearn.over_sampling import SMOTE
-from pathos.multiprocessing import ProcessingPool as Pool
-from pathos import multiprocessing
-from collections import Counter
-import numpy as np
+from cutoml.utils import timer
 import time
-import json
-import tqdm
 import warnings
 
+warnings.filterwarnings('ignore')
+
 
 class CutoClassifier:
-    def __init__(self, k_folds=3, oversample=False, n_jobs=2, verbose=0):
-        self.models = Classifiers(
-            k_folds=k_folds, n_jobs=n_jobs, verbose=verbose)
-        self.models = self.models.models
+    def __init__(self, k_folds=3, oversample=False, n_jobs=1, verbose=0, random_state=0):
+        self.random_state = random_state
+        classifiers_config = Classifiers(
+            k_folds=k_folds,
+            n_jobs=n_jobs,
+            verbose=verbose,
+            random_state=self.random_state
+        )
+        self.models = classifiers_config.models
         self.oversample = oversample
         self.best_estimator = None
         self.n_jobs = n_jobs
@@ -62,33 +59,44 @@ class CutoClassifier:
 
     def fit(self, X, y):
         X_train, X_test, y_train, y_test = train_test_split(
-            X, y, test_size=0.2, random_state=0
+            X, y, test_size=0.2, random_state=self.random_state
         )
 
         if self.oversample:
             over_sampler = SMOTE(
-                random_state=0,
+                random_state=self.random_state,
                 k_neighbors=3,
                 n_jobs=self.n_jobs
             )
-            X_train, y_train = over_sampler.fit_resample(X_train, y_train)
+            X_train, y_train = over_sampler.fit_resample(
+                X_train, y_train
+            )
 
         start_time = time.time()
-        pool = Pool()
-        try:
-            *trained_pipelines, = tqdm.tqdm(pool.imap(lambda x: self._model_fitter(x,
-                                                                                   X_train,
-                                                                                   y_train),
-                                                      self.models),
-                                            total=len(self.models),
-                                            desc='Optimization in progress')
-            end_time = time.time()
-        except KeyboardInterrupt:
-            pool.terminate()
-            pool.join()
-        finally:
-            pool.close()
-            pool.join()
+
+        pool = ProcessingPool()
+        with warnings.catch_warnings():
+            warnings.simplefilter("ignore")
+            try:
+                *trained_pipelines, = tqdm.tqdm(
+                    pool.imap(
+                        lambda x: self._model_fitter(x,
+                                                     X_train,
+                                                     y_train),
+                        self.models,
+                        chunksize=len(self.models) // 2
+                    ),
+                    total=len(self.models),
+                    desc='Optimizing'
+                )
+
+                end_time = time.time()
+            except KeyboardInterrupt:
+                pool.terminate()
+                pool.join()
+            finally:
+                pool.close()
+                pool.join()
         print(timer(start=start_time, end=end_time))
         trained_models = dict()
         for pipeline in trained_pipelines:
@@ -128,14 +136,19 @@ class CutoClassifier:
 
 
 class CutoRegressor:
-    def __init__(self, k_folds=3, n_jobs=2, verbose=0):
-        self.models = Regressors(
-            k_folds=k_folds, n_jobs=n_jobs, verbose=verbose)
-        self.models = self.models.models
+    def __init__(self, k_folds=3, n_jobs=1, verbose=0, random_state=0):
+        self.random_state = random_state
+        regressors_config = Regressors(
+            k_folds=k_folds,
+            n_jobs=n_jobs,
+            verbose=verbose,
+            random_state=self.random_state
+        )
+        self.models = regressors_config.models
         self.best_estimator = None
         self.n_jobs = n_jobs
 
-    def _model_fitter(sef, model, X, y):
+    def _model_fitter(self, model, X, y):
         try:
             rgr = Pipeline([
                 ('regression_model', model)
@@ -145,31 +158,38 @@ class CutoRegressor:
                 rgr = rgr.fit(X, y)
             return rgr
         except Exception as e:
+            print(e)
             pass
 
     def fit(self, X, y):
         X_train, X_test, y_train, y_test = train_test_split(
-            X, y, test_size=0.2, random_state=0
+            X, y, test_size=0.2, random_state=self.random_state
         )
         start_time = time.time()
-        pool = Pool()
-        try:
-            *trained_pipelines, = tqdm.tqdm(pool.imap(lambda x: self._model_fitter(x,
-                                                                                   X_train,
-                                                                                   y_train),
-                                                      self.models),
-                                            total=len(self.models),
-                                            desc='Optimization in progress'
-                                            )
-            end_time = time.time()
-        except KeyboardInterrupt:
-            pool.terminate()
-            pool.join()
-        finally:
-            pool.close()
-            pool.join()
-        # print(timer(start=start_time, end=end_time))
+        pool = ProcessingPool()
+        with warnings.catch_warnings():
+            warnings.simplefilter("ignore")
+            try:
+                *trained_pipelines, = tqdm.tqdm(
+                    pool.imap(
+                        lambda x: self._model_fitter(x,
+                                                     X_train,
+                                                     y_train),
+                        self.models,
+                        chunksize=len(self.models) // 2
+                    ),
+                    total=len(self.models),
+                    desc='Optimizing'
+                )
 
+                end_time = time.time()
+            except KeyboardInterrupt:
+                pool.terminate()
+                pool.join()
+            finally:
+                pool.close()
+                pool.join()
+        print(timer(start=start_time, end=end_time))
         trained_models = dict()
         for pipeline in trained_pipelines:
             if pipeline:
